﻿using AutoMapper;
using BL.DTOs;
using DAL;
using DAL.Entities;
using EFInfrastructure;
using EFInfrastructure.Helpers;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BL.QueryObjects
{
    public class PetQueryObject
    {
        private IMapper mapper;

        private EFQuery<Pet, int> myQuery;

        public PetQueryObject(IMapper mapper, PetsDbContext context)
        {
            this.mapper = mapper;
            myQuery = new EFQuery<Pet, int>(context);
        }

        public QueryResultDto<PetGetDto> ExecuteQuery(PetNameFilterDto filter)
        {
            var query = myQuery
                .Where(new MyPredicate(nameof(Pet.Name), ValueComparingOperator.Equal, filter.Name));
            if (!string.IsNullOrWhiteSpace(filter.SortCriteria))
            {
                query = query.SortBy(filter.SortCriteria, filter.SortAscending);
            }
            if (filter.RequestedPageNumber.HasValue)
            {
                query = query.Page(filter.RequestedPageNumber.Value, filter.PageSize);
            }

            return mapper.Map<QueryResultDto<PetGetDto>>(query.Execute());
        }
    }
}
