﻿namespace EFInfrastructure.Helpers
{
    public static class PredicateExtensions
    {
        private static readonly IDictionary<ValueComparingOperator, Func<object, string>> BinaryOperations =
            new Dictionary<ValueComparingOperator, Func<object, string>>
            {
                {ValueComparingOperator.Equal, rightOperand => CheckCommaUsage(rightOperand) ? $" = '{rightOperand}'" : $" = {rightOperand}" },
                {ValueComparingOperator.NotEqual, rightOperand => CheckCommaUsage(rightOperand) ? $" != '{rightOperand}'" : $" != {rightOperand}" },
                {ValueComparingOperator.GreaterThan, rightOperand => $" > {rightOperand}" },
                {ValueComparingOperator.LessThan, rightOperand => $" < {rightOperand}" }
            };

        public static string BuildSimplePredicate(this MyPredicate simplePredicate)
        {
            var predicate = simplePredicate;
            if (predicate == null)
            {
                throw new ArgumentException("Expected simple predicate!");
            }
            return GetWhereCondition(predicate);
        }
        private static string GetWhereCondition(MyPredicate simplePredicate)
        {
            if (!BinaryOperations.ContainsKey(simplePredicate.ValueComparingOperator))
            {
                throw new InvalidOperationException($"Transformation of value comparing operator: {simplePredicate.ValueComparingOperator} to where condition is not supported!");
            }
            if (simplePredicate.ComparedValue == null)
            {
                return GetNullWhereCondition(simplePredicate);
            }
            return GetEscapedWhereCondition(simplePredicate);
        }
        private static string GetNullWhereCondition(MyPredicate simplePredicate)
        {
            if (simplePredicate.ValueComparingOperator.Equals(ValueComparingOperator.NotEqual))
            {
                return simplePredicate.TargetPropertyName + " IS NOT NULL";
            }
            return simplePredicate.TargetPropertyName + " IS NULL";
        }
        private static bool CheckCommaUsage(object rightOperand)
        {
            return rightOperand != null && (rightOperand is string || rightOperand is Guid || rightOperand is DateTime);
        }
        private static string GetEscapedWhereCondition(MyPredicate simplePredicate)
        {
            const string atChar = "@";
            if (simplePredicate.ComparedValue is string value && value.Contains(atChar))
            {
                var escapedValue = (object)value.Insert(value.IndexOf(atChar, StringComparison.Ordinal), atChar);
                return simplePredicate.TargetPropertyName + BinaryOperations[simplePredicate.ValueComparingOperator]
                           .Invoke(escapedValue);
            }
            return simplePredicate.TargetPropertyName + BinaryOperations[simplePredicate.ValueComparingOperator]
                       .Invoke(simplePredicate.ComparedValue);
        }
    }
}
